// This Javascript is responsible for the display, design and functionalities of the calendar


// Function to fetch and process the .ics file
async function processICSFile() {
  try {
    // Fetch the .ics file
    const response = await fetch('/data/calendar.ics');
    if (!response.ok) {
      throw new Error('Failed to fetch the .ics file');
    }
    const data = await response.text();

    // Parse the .ics data
    const jcalData = ICAL.parse(data);
    const comp = new ICAL.Component(jcalData);

    // Initialize an array to hold events for FullCalendar
    let fullCalendarEvents = [];

    // Process each VEVENT component
    const vevents = comp.getAllSubcomponents('vevent');
    vevents.forEach(veventComp => {
      const vevent = new ICAL.Event(veventComp);

      // Skip all-day events
      if (vevent.component.getFirstPropertyValue('x-microsoft-cdo-alldayevent') === 'TRUE') {
        return;
      }

      // Handle recurring events
      if (vevent.isRecurring()) {
        const exdates = getExdates(vevent);
        const rrule = vevent.component.getFirstPropertyValue('rrule');
        
        // Adjust the UNTIL property to include the entire day
        if (rrule && rrule.until) {
            rrule.until = adjustUntilDate(rrule.until);
        }

        let iterator = vevent.iterator();
        let next;
        while ((next = iterator.next())) {
            let occurrenceDate = next.toJSDate();
            if (!exdates.has(occurrenceDate.toISOString().split('T')[0])) {
                fullCalendarEvents.push(formatEventForFullCalendar(vevent, occurrenceDate));
            }
        }
    } else {
        // Non-recurring events
        fullCalendarEvents.push(formatEventForFullCalendar(vevent, vevent.startDate.toJSDate()));
      }
    });

    // Filter and replace occurrences with exceptions
    return processExceptions(fullCalendarEvents);
  } catch (error) {
    console.error('Error processing .ics file:', error);
  }
}

function adjustUntilDate(until) {
  // Set the time to the end of the day to include the entire day
  until.hour = 23;
  until.minute = 59;
  until.second = 59;
  return until;
}

function getExdates(vevent) {
  let exdates = new Set();
  vevent.component.getAllProperties('exdate').forEach(exdateProp => {
    exdateProp.getValues().forEach(exdate => {
      exdates.add(exdate.toJSDate().toISOString().split('T')[0]);
    });
  });
  return exdates;
}

function formatEventForFullCalendar(vevent, occurrenceDate) {
  return {
    title: vevent.summary,
    start: occurrenceDate.toISOString(),
    end: new Date(occurrenceDate.getTime() + (vevent.endDate.toJSDate() - vevent.startDate.toJSDate())).toISOString(),
    uid: vevent.uid,
    recurrenceId: vevent.component.getFirstPropertyValue('recurrence-id')?.toJSDate().toISOString().split('T')[0]
  };
}

function processExceptions(events) {
  let eventMap = new Map();

  // First pass: Add all events to the map, exceptions will replace regular events
  events.forEach(event => {
      let date = event.recurrenceId || event.start.split('T')[0];
      let key = date + "-" + event.uid;
      eventMap.set(key, event);
  });

  // Filter the map to ensure only one occurrence per UID per date
  let finalEvents = Array.from(eventMap.values()).filter((event, _, self) => {
      let date = event.start.split('T')[0];
      let matchingEvents = self.filter(e => e.start.split('T')[0] === date && e.uid === event.uid);

      // Keep the event if it's the only one for this UID on this date, or if it's an exception
      return matchingEvents.length === 1 || event.recurrenceId;
  });
  // console.log(JSON.stringify(finalEvents, null, 2));
  return finalEvents;
}


// Call the function to process the .ics file
processICSFile().then(finalEvents => {


  // Sort events by start date
  const sortedEvents = finalEvents.sort((a, b) => new Date(a.start) - new Date(b.start));

  // Group by month (YYYY-MM)
  const eventsByMonth = {};
  for (const event of sortedEvents) {
    const eventDateObj = new Date(event.start);
    const year = eventDateObj.getFullYear();
    const month = (eventDateObj.getMonth() + 1).toString().padStart(2, '0');
    const monthKey = `${year}-${month}`;

    if (!eventsByMonth[monthKey]) {
      eventsByMonth[monthKey] = [];
    }
    eventsByMonth[monthKey].push(event);
  }

  // Prepare date comparisons
  const now = new Date();
  const todayMidnight = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0);

  // Function to format time in German locale, HH:MM
  function formatTime(date) {
    return date.toLocaleTimeString('de-DE', { hour: '2-digit', minute: '2-digit' });
  }

  // Get an array of all month keys sorted chronologically
  const monthKeys = Object.keys(eventsByMonth).sort((a, b) => {
    const [yearA, monthA] = a.split('-').map(Number);
    const [yearB, monthB] = b.split('-').map(Number);
    return new Date(yearA, monthA - 1) - new Date(yearB, monthB - 1);
  });

  // We'll build monthContainers and keep track of each month's name as well
  const monthContainers = [];
  const monthNames = [];

  // Build monthContainers but do not append yet
  for (const monthKey of monthKeys) {
    const [year, month] = monthKey.split('-').map(Number);
    const monthDate = new Date(year, month - 1);
    const monthName = monthDate.toLocaleString('de-DE', { month: 'long', year: 'numeric' });

    // Group events by date within this month
    const eventsByDate = {};
    eventsByMonth[monthKey].forEach(event => {
      const eventDate = event.start.split('T')[0]; // YYYY-MM-DD
      if (!eventsByDate[eventDate]) {
        eventsByDate[eventDate] = [];
      }
      eventsByDate[eventDate].push(event);
    });

    // Create the month container (without a month header)
    const monthContainer = document.createElement('div');
    monthContainer.className = 'cal_month';

    let monthHasVisibleEvents = false;

    Object.keys(eventsByDate).forEach(dateKey => {
      const dateEvents = eventsByDate[dateKey];
      const dateContainer = document.createElement('div');
      dateContainer.className = 'cal_date';
    
      // Convert YYYY-MM-DD to a German-formatted date and weekday
      const [dYear, dMonth, dDay] = dateKey.split('-').map(Number);
      const dateObj = new Date(dYear, dMonth - 1, dDay);
    
      // Separate weekday and date
      const weekdayLocalized = dateObj.toLocaleDateString('de-DE', { weekday: 'long' });
      const dateLocalized = dateObj.toLocaleDateString('de-DE', { year: 'numeric', month: 'long', day: 'numeric' });
    
      // Date header (h4)
      const dateHeader = document.createElement('h4');
      dateHeader.className = 'cal_date__header';
    
      // Create two spans: left (date) and right (weekday)
      const dateLeftSpan = document.createElement('span');
      dateLeftSpan.className = 'cal_date__header_left';
      dateLeftSpan.textContent = dateLocalized;
    
      const dateRightSpan = document.createElement('span');
      dateRightSpan.className = 'cal_date__header_right';
      dateRightSpan.textContent = weekdayLocalized;
    
      dateHeader.appendChild(dateLeftSpan);
      dateHeader.appendChild(dateRightSpan);
    
      dateContainer.appendChild(dateHeader);
    
      // List of events for this date
      const eventList = document.createElement('ul');
      let dateHasVisibleEvents = false;
    
      for (const event of dateEvents) {
        const eventItem = document.createElement('li');
        eventItem.className = 'cal_event hover'; 
        
        // Store the event data as a JSON string in the data-event attribute
        eventItem.setAttribute('data-event', JSON.stringify(event));
        
        const eventEnd = new Date(event.end);
        if (eventEnd < todayMidnight) {
          // Past event
          eventItem.classList.add('cal_event__past');
        } else {
          // Future or ongoing event - mark the date as having visible events
          dateHasVisibleEvents = true;
        }
      
        // Format time range
        const startTime = new Date(event.start);
        const endTime = new Date(event.end);
        const timeRange = `${formatTime(startTime)} - ${formatTime(endTime)}`;
      
        eventItem.textContent = `${timeRange} | ${event.title}`;
        eventList.appendChild(eventItem);
      }
      
      if (dateHasVisibleEvents) {
        dateContainer.appendChild(eventList);
        monthContainer.appendChild(dateContainer);
        monthHasVisibleEvents = true;
      }
    });

    if (monthHasVisibleEvents) {
      monthContainers.push(monthContainer);
      monthNames.push(monthName);
    }
  }

  // Determine which month to show initially
  // First, try to find the current month (YYYY-MM)
  const currentYear = now.getFullYear();
  const currentMonth = (now.getMonth() + 1).toString().padStart(2, '0');
  const currentMonthKey = `${currentYear}-${currentMonth}`;

  let currentMonthIndex = monthKeys.indexOf(currentMonthKey);

  // If no events in the current month, find the closest future month with events
  if (currentMonthIndex === -1 || !monthKeys[currentMonthIndex] || !monthContainers[monthKeys.indexOf(currentMonthKey)]) {
    // Find the first monthKey that is >= currentMonthKey and has a container
    let foundIndex = monthKeys.findIndex((mk, i) => {
      const [y,m] = mk.split('-').map(Number);
      const mkDate = new Date(y, m-1);
      const currentDate = new Date(currentYear, Number(currentMonth)-1);
      return (mkDate >= currentDate && monthContainers[monthKeys.indexOf(mk)]); 
    });

    // If not found any future month, use the earliest available month with events
    if (foundIndex === -1) {
      // map monthKeys to indices of monthContainers
      foundIndex = monthKeys.findIndex(mk => monthKeys.indexOf(mk) < monthContainers.length && monthKeys.indexOf(mk) !== -1 && monthContainers[monthKeys.indexOf(mk)]);
      if (foundIndex === -1) {
        foundIndex = 0; // fallback if something's off
      }
    }

    currentMonthIndex = foundIndex;
  }


  const visibleMonthIndices = monthKeys.reduce((acc, mk, idx) => {
    const containerIdx = monthContainers.indexOf(
      monthContainers.find(mc => {
        const [y,mo] = mk.split('-').map(Number);
        const testMonth = new Date(y, mo - 1);
        const testName = testMonth.toLocaleString('de-DE', { month: 'long', year: 'numeric' });
        return mc.querySelector('.cal_date') !== null && monthNames[monthContainers.indexOf(mc)] === testName;
      })
    );
    if (containerIdx !== -1) acc.push({ mk, containerIdx });
    return acc;
  }, []);

  const visibleMonthIndexMap = new Map(visibleMonthIndices.map(obj => [obj.mk, obj.containerIdx]));

  // If currentMonthIndex doesn't point to a visible month directly, adjust it:
  if (!visibleMonthIndexMap.has(monthKeys[currentMonthIndex])) {
    // If the current month key isn't found in the visible map (no events?), pick the first visible month
    currentMonthIndex = visibleMonthIndices[0].containerIdx;
  } else {
    currentMonthIndex = visibleMonthIndexMap.get(monthKeys[currentMonthIndex]);
  }

  // Create main calendar container if not existing
  const calendarContainer = document.getElementById('calendar');
  const controlsContainer = document.createElement('div');
  controlsContainer.className = 'cal_controls';

  const prevButton = document.createElement('button');
  prevButton.className = 'cal_controls__prev hover';
  prevButton.textContent = '◀';

  const monthTitle = document.createElement('h3'); 
  monthTitle.className = 'cal_month__header';

  const nextButton = document.createElement('button');
  nextButton.className = 'cal_controls__next hover';
  nextButton.textContent = '▶';

  controlsContainer.appendChild(prevButton);
  controlsContainer.appendChild(monthTitle);
  controlsContainer.appendChild(nextButton);
  calendarContainer.appendChild(controlsContainer);

  // showMonth function that displays the chosen month
  function showMonth(index) {
    // Remove existing month (if any)
    const existingMonth = calendarContainer.querySelector('.cal_month');
    if (existingMonth) {
      calendarContainer.removeChild(existingMonth);
    }

    // Show the requested month if it exists
    if (monthContainers[index]) {
      monthTitle.textContent = monthNames[index];
      calendarContainer.appendChild(monthContainers[index]);
      
      // Add click handlers to events
      const eventItems = monthContainers[index].querySelectorAll('.cal_event');
      eventItems.forEach(eventItem => {
        eventItem.addEventListener('click', () => {
          const eventData = JSON.parse(eventItem.getAttribute('data-event'));
          showContactForm(eventData);
        });
      });
    }
  }

  // Initial display
  showMonth(currentMonthIndex);

  // Navigation
  prevButton.addEventListener('click', () => {
    if (currentMonthIndex > 0) {
      currentMonthIndex--;
      showMonth(currentMonthIndex);
    }
  });

  nextButton.addEventListener('click', () => {
    if (currentMonthIndex < monthContainers.length - 1) {
      currentMonthIndex++;
      showMonth(currentMonthIndex);
    }
  });
});

// This will hold the dynamically inserted form.
const formWrapper = document.getElementById('form-wrapper');

// Comment: Utility function to format dates/times in German locale
function formatGermanDate(date) {
  return new Intl.DateTimeFormat('de-DE', {
    year: 'numeric',
    month: 'long',
    day: '2-digit'
  }).format(date);
}

function formatGermanTime(date) {
  return new Intl.DateTimeFormat('de-DE', {
    hour: '2-digit',
    minute: '2-digit',
    hour12: false
  }).format(date);
}

/**
 * Displays a contact form for a given event at the top of the page,
 * scrolling to it and allowing the user to sign up.
 * 
 * @param {Object} event - The event object with {title, start, end, ...}.
 */
function showContactForm(event) {
  // Convert start and end to Date objects
  let startDate = new Date(event.start);
  let endDate = new Date(event.end);

  // Determine if the event is in the past. Using today's midnight to compare just dates.
  const now = new Date();
  const todayMidnight = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0);
  
  if (startDate < todayMidnight) {
    alert("Du kannst Dich nicht für ein Ereignis in der Vergangenheit anmelden.");
    return;
  }

  // Format the date and times using German locale
  let formattedDate = formatGermanDate(startDate);
  let formattedStartTime = formatGermanTime(startDate);
  let formattedEndTime = formatGermanTime(endDate);

  // Generate the HTML for the event form
  const formHTML = generateEventHTML(event, formattedDate, formattedStartTime, formattedEndTime);

  // Clear any existing form
  formWrapper.innerHTML = '';

  // Insert the new form into the form-wrapper
  formWrapper.innerHTML = formHTML;

  // Get references to the newly inserted form and close button
  const eventForm = document.getElementById('event-form');
  const closeButton = document.getElementById('close-button');

  add_spam_protect();

  // Show the form
  eventForm.style.display = 'block';

  // Scroll to the form
  eventForm.scrollIntoView({ behavior: 'smooth', block: 'start' });

  // Handle the close button click
  closeButton.addEventListener('click', () => {
    // Hide the form
    eventForm.style.display = 'none';
  });
}

/**
 * Generates the HTML string for the event form.
 *
 * @param {Object} event - The event object.
 * @param {string} formattedDate - The formatted event date.
 * @param {string} formattedStartTime - The formatted start time.
 * @param {string} formattedEndTime - The formatted end time.
 * @returns {string} HTML string of the form.
 */
function generateEventHTML (event, formattedDate, formattedStartTime, formattedEndTime) {
  return `
    <form id="event-form" class="form" style="display: none;" method="post" action="https://feedback.ruhr-uni-bochum.de/formmailer.php">
      <input type="hidden" name="subject" value="Teilnahmeanfrage von der Makerspace-Homepage" /> 
      <div class="event-subject">
        <label for="event-info"><strong>Deine Teilnahmeanfrage</strong></label>
        <input type="hidden" name="event_info" id="event-info-input" 
               value="${event.title + '\nam ' + formattedDate + ' von ' + formattedStartTime + ' bis ' + formattedEndTime + ' Uhr'}" />
        <div id="event-info-display" readonly> 
          <p>Event</p>
          <p>Datum</p>
          <p>Uhrzeit</p>
          <p style="font-weight: 600">${event.title}</p>
          <p style="font-weight: 600">${formattedDate}</p>
          <p style="font-weight: 600">${formattedStartTime} - ${formattedEndTime}</p>
        </div>
      </div>   
      <input type="hidden" name="recipients" value="makerspace@ruhr-uni-bochum.de"/>
      <input type="hidden" name="mail_options" value="charset=UTF-8,Exclude=realname;honigtopf;schicken;FromAddr"/>
      <input type="hidden" name="good_url" value="https://makerspace.ruhr-uni-bochum.de/danke/"  />       

      <div class="form-field">
        <label for="nachname"><strong>Name:</strong></label>
        <input name="nachname" id="nachname" type="text" maxlength="30" required/>
      </div>							

      <div class="form-field">
        <label for="vorname"><strong>Vorname:</strong></label>
        <input name="vorname" id="vorname" type="text" maxlength="30" required/>
      </div>							

      <div class="form-field">
        <label for="email"><strong>E-Mail:</strong></label>
        <input name="email" id="email" type="email" maxlength="60" required/>
      </div>			

      <div class="form-field">
        <label for="tel"><strong>Telefonnummer:</strong></label>
        <input name="tel" id="tel" type="tel" maxlength="40" placeholder="Optional" />
      </div>
      
      <div class="form-field">
        <label for="rolle"><strong>Deine Rolle:</strong></label>
        <select name="rolle" id="rolle" style="font-size: 1em; padding: 0.5em;" required>
          <option value="" disabled selected>Bitte wählen</option>
          <option value="Worldfactory-Gründungsteam oder -Mitarbeiter*in">Worldfactory-Start-up</option>
          <option value="Start-up oder Gründungsteam (ohne Worldfactory-Bezug)">Externes Start-up</option>
          <option value="RUB-Angehörige*r (ohne Worldfactory-Bezug)">RUB-Angehörige*r</option>                    
          <option value="Unternehmen / Wirtschaftliche Einrichtung">Unternehmen</option>
          <option value="Privatperson">Privatperson</option>
          <option value="Öffentliche Einrichtung ohne RUB-Zugehörigkeit">Öffentliche Einrichtung</option>
        </select>
        <span>Mehr Informationen zu unseren Zielgruppen findest Du <a href="/ms-nutzen#ziel" target="_blank" style="color: var(--link-color);">hier<span style="display: inline-flex;height: 1.125em;vertical-align: text-top;color: var(--link-color);"><svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 16 16"><path d="M3.75 2h3.5a.75.75 0 0 1 0 1.5h-3.5a.25.25 0 0 0-.25.25v8.5c0 .138.112.25.25.25h8.5a.25.25 0 0 0 .25-.25v-3.5a.75.75 0 0 1 1.5 0v3.5A1.75 1.75 0 0 1 12.25 14h-8.5A1.75 1.75 0 0 1 2 12.25v-8.5C2 2.784 2.784 2 3.75 2m6.854-1h4.146a.25.25 0 0 1 .25.25v4.146a.25.25 0 0 1-.427.177L13.03 4.03 9.28 7.78a.75.75 0 0 1-1.042-.018.75.75 0 0 1-.018-1.042l3.75-3.75-1.543-1.543A.25.25 0 0 1 10.604 1"/></path></svg></span></a>.</span>
      </div>				

      <div class="form-field">
        <label for="nachricht" id="nachricht-text"><strong>Dein Anliegen:</strong><span class='large'></span></label>
        <textarea name="nachricht" id="nachricht" maxlength="500" placeholder="Bitte beschreibe hier kurz Dein Anliegen. (Limit: 500 Zeichen)" rows="6"></textarea>
      </div> 

      <div class="honey" style="display: none;">
        <label for="honigtopf"><strong>Please do not fill this field:</strong></label>
        <input name="honigtopf" id="honigtopf" type="text" maxlength="50" value="" />
      </div>	

      <p style="font-size: var(--form-note-font-size);">Bitte beachte: Diese Anfrage allein berechtigt Dich noch nicht zur Teilnahme. Die Teilnahme ist erst nach Erhalt einer Bestätigungsmail von uns gültig!</p>

      <div class="form-field" style="margin-bottom: 10px; font-size: var(--form-note-font-size);">
        <label for="data-privacy" style="font-size: var(--form-note-font-size);">
          <div style="float: left; padding: 0px;">
            <input type="checkbox" id="data-privacy" name="data-privacy" required>
          </div>
          <div style="overflow: hidden; padding-left: 10px;">
            Ich erkläre mich mit der Erhebung, Verarbeitung und Speicherung meiner hier gemachten Angaben gemäß Eurer
            <a href="https://www.ruhr-uni-bochum.de/de/datenschutz" target="_blank" style="color: var(--link-color);">Datenschutzvereinbarung</a> einverstanden. Meine Einwilligung kann ich jederzeit durch formlose Mitteilung an Euch widerrufen.
          </div>
        </label>
      </div>
  
      <div id="button-wrapper">
        <div class="form-field">
          <input type="submit" id="submit-button" class="md-button hover" value="Absenden" name="schicken" />  
        </div>
        <div class="form-field">
          <input type="button" id="close-button" class="md-button hover" value="Abbrechen" name="abbrechen" />
        </div>
      </div>				
    </form>
  `;
}

/** 
 * Spam protection:
 * Checks if 'honigtopf' has been filled. 
 * If yes, rejects submission.
 * 
 * Also, checks if at least 10 seconds has passed to fill out the form.
 * If not, rejects submission
 */ 
function add_spam_protect(){
  document.getElementById("event-form").addEventListener("submit", function (e) {
    const honeypot = document.querySelector('input[name="honigtopf"]');
    if (honeypot.value !== "") {
      e.preventDefault();
      alert("Spam detected. Submission rejected.");
    }
  });
  const form = document.getElementById("event-form");
  const submitTime = new Date().getTime();

  form.addEventListener("submit", function (e) {
    const currentTime = new Date().getTime();
    const timeDifference = (currentTime - submitTime) / 1000;

    if (timeDifference < 10) { 
      e.preventDefault();
      alert("Please take more time to fill out the form.");
    }
  });
};